--[[
	Loads all background music
	Caveware Digital :: 2017
]]


--[[
	Recursively checks a folder for sounds and adds any found to a table
	INPUT: Directory and table or nil
	OUTPUT: Table of loaded assets
]]
local function loadBGMFolder(dir, tab)
	-- Create new table or use given
	local tab = tab or newExtendedTable({})

	-- Get a table of files / subdirs from dir
	local items = love.filesystem.getDirectoryItems(dir)

	for key, val in ipairs(items) do
		if (love.filesystem.isFile(dir .. "/" .. val)) then
			if (val ~= "init.lua") then
				-- Remove ".ogg" extension on file
				name = val:sub(1, -5)

				-- Load sound into table
				tab[name] = love.audio.newSource(dir .. "/" .. val, 'stream')
				tab[name]:setLooping(true)
			end
		else
			-- Add new table onto tab
			tab[val] = newExtendedTable({})

			-- Run checkFolder in this subdir
			loadBGMFolder(dir .. "/" .. val, tab[val])
		end
	end

	return tab
end


-- Load the bgm folder
local bgm = loadBGMFolder(...)
bgm.volume = 1


--[[
	Gets volume for all BGM
	INPUT: Volume
]]
function bgm:getVolume()
	return self.volume
end


--[[
	Sets volume for all BGM
	INPUT: Volume
]]
function bgm:setVolume(volume)
	self.volume = volume

	for key, sound in ipairs(self) do
		sound:setVolume(volume)
	end
end


--[[
	Stops all BGM
]]
function bgm:stopAll()
	for key, sound in ipairs(self) do
		sound:stop()
	end
end


return bgm