--[[
	Loads all graphics assets
	Caveware Digital :: 2017
]]


--[[
	Recursively checks a folder for graphics and adds any found to a table
	INPUT: Folder to load and table or nil
	OUTPUT: Table of loaded assets
]]
local function loadGFXFolder(folder, tab)
	-- Table is either new table or given
	local tab = tab or newExtendedTable({})

	-- Obtain table of items in directory
	local items = love.filesystem.getDirectoryItems(folder)

	-- Iterate through all items in directory
	for key, item in ipairs(items) do
		-- Check if item is a file or folder and act accordingly
		if (love.filesystem.isFile(folder .. "/" .. item)) then
			-- Split item into name and type of file
			local name, ext =
				item:sub(1, -5),
				item:sub(-3)

			-- If item is either jpg or png...
			if (ext == "jpg" or ext == "png") then

				-- ...load image into table
				tab[name] = love.graphics.newImage(folder .. "/" .. item)
			end
		else
			-- Create new table and load folder recursively
			tab[item] = {}
			loadGFXFolder(folder .. "/" .. item, tab[item])
		end
	end

	-- Return loaded table
	return tab
end


-- Load the gfx folder
return loadGFXFolder(...)