-- Rolled my own map tool, here goes

local current_dir = ...
-- Hardloading the test map for test purposes
walls		= {}
local map = {}

-- pathfinding variables
level_mapnodes  = newExtendedTable({})
local graph_flag = {}

-- Load map objects as walls
function map:load()
	tiles = require("/assets/maps/test")
	for k, v in ipairs(tiles.layers[1].objects) do
		if v.shape == "rectangle" then
			table.insert(walls, {
				s		= v.shape,
				x		= v.x,
				y		= v.y,
				w		= v.width,
				h		= v.height
			})
			-- add flag for pathfinding detection
			for i = 1, (v.width/32) do
				for j = 1, (v.height/32) do
					local x = v.x + (i - 1) * 32
					local y = v.y + (j - 1) * 32
					local f = map:flagEmpty(x, y, true)
					table.insert(graph_flag, f)
					print("graph_flag size: " .. #graph_flag .. " at pos: ")
					print(x, "	", y)
				end
			end


			--[[walls.body	= lp.newBody(World, v.x - v.width / 2, v.y - v.height / 2)
			walls.shape	= lp.newRectangleShape(v.width, v.height)
			walls.fixture	= lp.newFixture(wall.body, wall.shape)]]--
		elseif v.shape == "polygon" then
			local points = {}
			for key, point in ipairs(v.polygon) do
				table.insert(points, point.x + v.x)
				table.insert(points, point.y + v.y)
			end
			table.insert(walls, {
				s			= v.s,
				x			= v.x,
				y			= v.y,
				vertices	= points
			})
		end
	end
	for k, v in ipairs(walls) do
		if v.s == "rectangle" then
			v.body		= lp.newBody(World, v.x - v.w / 2, v.y - v.h / 2, "dynamic")
			v.shape		= lp.newRectangleShape(v.x, v.y, v.w, v.h)
			v.fixture	= lp.newFixture(v.body, v.shape)
		elseif v.s == "polygon" then
			v.body		= lp.newBody(World, v.x - 16, v.y - 16, "dynamic")
			v.shape		= lp.newPolygonShape(v.vertices)
			v.fixture	= lp.newFixture(wall.body, wall.shape)
		end
	end
	map:createGraph()
end

function map:draw()
	lg.setColor(255,255,255)
	for key, wall in ipairs(walls) do
		if wall.s == "rectangle" then
			lg.rectangle("fill", wall.x, wall.y, wall.w, wall.h)
		elseif wall.s == "polygon" then
			test = wall.shape:getPoints()
			print(test)
			lg.polygon("fill", wall.shape:getPoints())
		end
	end
	if (Flags.DEVELOPMENT) then
		map:testGraphDraw(graph)
	end
end

-- creates the graph of nodes used for a* pathfinding. run after map.load
function map:createGraph()
	for i = 1, love.graphics.getWidth()/32 do
		for j = 1, love.graphics.getHeight()/32 do
			local wall_detected = false
			local x = (i - 1) * 32
			local y = (j - 1) * 32
			for i, v in ipairs(graph_flag) do
				if v.x == x and v.y == y and v.empty then
					wall_detected = true
				end
			end
			if not wall_detected then
				local m = map:addGraphNode(x, y)
				table.insert(level_mapnodes, m)
			end
		end
	end
end

-- flags a grid square as empty during map load for pathfinding graph
function map:flagEmpty(x, y, bool)
	local coord = {}
	coord.x = x
	coord.y = y
	coord.empty = bool
	return coord
end

-- adds a node to the graph. includes a unique id and references the centre of the graph square, rather than it x, y location
function map:addGraphNode(x, y)
	local coord = {}
	coord.id = #level_mapnodes + 1
	coord.x = x + 16
	coord.y = y + 16
	return coord
end

function map:testGraphDraw(graph)
	love.graphics.setColor(ColorTool:get("blue"))
	for i, v in ipairs(level_mapnodes) do
		love.graphics.rectangle("fill", v.x - 1, v.y - 1, 3, 3)
	end
end

return map
