--[[
	The class for all game states
	Caveware Digital :: 2017
]]

local  Enemy = {}
Enemy.__index =  Enemy


--[[
	Create a game state
	OUTPUT: The newly created game state
]]
function Enemy:new(source)
	local enemy = {}
	setmetatable(enemy, self)

	enemy.x = source.x or 200
	enemy.y = source.y or 200
	enemy.r = source.r or 2

	enemy.speed = 400
	enemy.step_int = 50
	enemy.timer = 50

	enemy.x_dest = nil
	enemy.y_dest = nil

	enemy.enemy_waves = {}

	enemy.path = source.path
	enemy.behavior = source.behavior
	Enemy:newBehavior(enemy.path, enemy.behavior)

	return enemy
end

function Enemy:getCollisionShape()
	return {x = self.x, y = self.y, r = self.r}
end

--[[
	Performed to draw the game state
]]
function Enemy:draw()
	if (Flags.DEVELOPMENT) then
		love.graphics.setColor(ColorTool:get("red"))
		love.graphics.rectangle("fill", self.x - 1, self.y - 1 , 2, 2)
	end
	for i, v in ipairs(self.enemy_waves) do
		v:draw()
	end
end


--[[
	Performed when the game state is paused
]]
function Enemy:pause() end


--[[
	Performed when the game state is resumed
]]
function Enemy:resume() end


--[[
	Performed to start the game state
]]
function Enemy:start(options) end


--[[
	Performed to destroy the game state (eg. moving from level-to-level)
]]
function Enemy:stop() end


--[[
	Performed to update the game state (each step it's active)
]]
function Enemy:update(dt)
	if self.x_dest or self.y_dest then
		if self.timer >= self.step_int then
			local x = self.x_dest or self.x
			local y = self.y_dest or self.y
			local direction = math.angle(self.x, self.y, x, y)
			local dx = math.cos(direction) * self.speed * dt
			local dy = math.sin(direction) * self.speed * dt
			self.x = self.x + dx
			self.y = self.y + dy
			table.insert(self.enemy_waves, Wave:new( { x = self.x, y = self.y, color = "red", max_dist = 80} ))
			self.timer = 0
		else
			self.timer = self.timer + 60*dt
		end
	end

	for i, v in ipairs(self.enemy_waves) do
		v:update(dt)
	end
end

function Enemy:heardNoise(x, y)
	self.x_dest = x
	self.y_dest = y
end
function Enemy:noNoise()
	self.x_dest = nil
	self.y_dest = nil
end

-- currently only supports a simple start-end patrol
function Enemy:newBehavior(path, behavior, mapnodes)
	if behavior == "patrol" then
		local path_start = {
			x = path[1],
			y = path[2]
		}
		local path_end = {
			x = path[3],
			y = path[4]
		}

		local valid_node_func = function ( node, neighbor )

		    local MAX_DIST = 32

		    -- helper function in the a-star module, returns distance between points
		    if astar.distance ( node.x, node.y, neighbor.x, neighbor.y ) < MAX_DIST then
		        return true
		    end
		    return false
		end

		local ignore = true
		--local path = astar.path (path_start, path_end, mapnodes, ignore, valid_node_func)
	end
	return false
end



return  Enemy
