--[[
	Particle class: holds and manages the behavior of individual particles
	Caveware Digital :: 2017
]]


local Particle = {}
Particle.__index = Particle


--[[
	Create a game state
	OUTPUT: The newly created game state
]]
function Particle:new(source)
	local particle = {}
	setmetatable(particle, self)
	particle.max_dist = source.max_dist --source.speed
	particle.distance = source.travelled_total or 0 -- total distance travelled
	particle.speed = source.speed or 140
	particle.w = 3
	particle.h = 3
	particle.alpha_scale = 1
	particle.x = source.x
	particle.y = source.y

	-- these will change on collision
	particle.origin_x = source.x_o
	particle.origin_y = source.y_o
	particle.origin_distance = source.travelled_from_origin or 0
	particle.theta = source.theta

	particle.sneak = source.sneak
	particle.color = source.color
	particle.colortable = ColorTool:get(particle.color)
	if particle.sneak then
		particle.alpha_max = 120
	else
		particle.alpha_max = 255
	end
	particle.alpha = source.alpha or particle.alpha_max

	particle.body		=	love.physics.newBody(World, particle.x - particle.w / 2, particle.y - particle.h / 2, "kinematic")
	particle.shape		=	love.physics.newRectangleShape(particle.w, particle.h)
	particle.fixture	=	love.physics.newFixture(particle.body, particle.shape)
	particle.body:setLinearVelocity(particle.speed * math.cos(particle.theta), particle.speed * math.sin(particle.theta))


	return particle
end


function Particle:getCollisionShape()
	return {x = self.x, y = self.y, w = self.w, h = self.h}
end

--[[
	Performed to draw the game state
]]
function Particle:draw()
	love.graphics.setColor(self.colortable[1], self.colortable[2], self.colortable[3], self.alpha)
	love.graphics.rectangle("fill", self.x, self.y, 3, 3)
 end


--[[
	Performed when the game state is paused
]]
function Particle:pause() end


--[[
	Performed when the game state is resumed
]]
function Particle:resume() end


--[[
	Performed to start the game state
]]
function Particle:start(options) end


--[[
	Performed to destroy the particle
]]
function Particle:stop()

end


--[[
	Performed to update the particle (each step it's active)
]]
function Particle:update(dt)

	local vx, vy = self.body:getLinearVelocity()
	self.x = self.x + vx * dt
	self.y = self.y + vy * dt
	self.distance = self.distance + self.speed * dt
	return self.distance




end


function Particle:getOrigin()
	return self.origin_x, self.origin_y
end

function Particle:initialisePosition(newTheta)
	local x = self.origin_x + self.origin_distance * math.cos(math.rad(newTheta))
	local y = self.origin_y + self.origin_distance * math.sin(math.rad(newTheta))
	return x, y
end



return Particle
