--[[
	The class for all game states
	Caveware Digital :: 2017
]]


local SoundMaker = {}
SoundMaker.__index = SoundMaker


--[[
	Create a game state
	OUTPUT: The newly created game state
]]
function SoundMaker:new(source)
	local sounder = {}
	setmetatable(sounder, self)
	sounder.x = source.x
	sounder.y = source.y
	sounder.r = source.r -- range of interactions

	sounder.timer = 50 --counter
	sounder.interval = 50 --interval of time between waves

	sounder.interact = false --if object(player) is inside sound radius

	sounder.color = source.color or "blue"

	sounder.obj_waves = {} -- holds wave objects

	return sounder
end


--[[
	Performed to draw the game state
]]
function SoundMaker:draw()
	--draws if objectWithinRadius gives true
	if self.interact then
		for i, v in ipairs(self.obj_waves) do
			v:draw()
		end
	end
end


--[[
	Performed when the game state is paused
]]
function SoundMaker:pause() end


--[[
	Performed when the game state is resumed
]]
function SoundMaker:resume() end


--[[
	Performed to start the game state
]]
function SoundMaker:start(options) end


--[[
	Performed to destroy the game state (eg. moving from level-to-level)
]]
function SoundMaker:stop() end


--[[
	Performed to update the game state (each step it's active)
]]
function SoundMaker:update(dt)
	if self.interact then
		--checks for interval to make make a wave
		if self.timer >= self.interval then
			table.insert(self.obj_waves, Wave:new( { x = self.x, y = self.y, color = self.color, max_dist = 45} ))
			self.timer = 0
		else
			self.timer = self.timer + 60*dt
		end

		for i, v in ipairs(self.obj_waves) do
			v:update(dt)
		end
	end
end

--takes a table of the objects x,y,width(w) and height(h) to determine if it is
--within the range(self.r) of this object
function SoundMaker:objWithinRadius(t)
	local within = false
	local points = {{x=t.x,y=t.y}, {x=t.x+t.w,y=t.y}, {x=t.x+t.w,y=t.y+t.h}, {x=t.x,y=t.y+t.h}}
	for i,v in ipairs(points) do
		if math.distance(v.x, v.y, self.x, self.y) <= self.r then
			within = true
		end
	end
	self.interact = within
end

function SoundMaker:obtain(x, y)
	if math.distance(x,y,self.x,self.y) < 15 then
		self.interact = false
		return true
	else
		return false
	end
end

return SoundMaker
