--[[
	initial game state for ggj 2017
	Caveware Digital :: 2017
]]

require("helpers")
levels = require(... .. "/levels")

local Game = {}
Game.__index = Game


--[[
	Create a game state
	OUTPUT: The newly created game state
]]
function Game:new()
	local game_state = {}
	setmetatable(game_state, self)
	return game_state
end


--[[
	Performed to draw the game state
]]
function Game:draw()
	for i,enemy in ipairs(self.enemies) do
		enemy:draw()
	end
	self.player:draw()
	for i,noise in ipairs(self.noises) do
		noise:draw()
	end
	if not self.player:gotDucky() then
		self.rubber_ducky:draw()
	end
	self.door:draw()
	map:draw()
end


--[[
	Performed when the game state is paused
]]
function Game:pause() end


--[[
	Performed when the game state is resumed
]]
function Game:resume() end


--[[
	Performed to start the game state
]]
function Game:start(options)
	self.level = levels:load(options)
	self.level:makeLevel()
	map:load()
	local prop =  self.level:getProperties()
	self.player = player:new{x = prop.player_start.x, y = prop.player_start.y}
	self.enemies = {}
	for i, enemy in ipairs(prop.enemy_list) do
		local path = {enemy.x, enemy.y, enemy.dest_x, enemy.dest_y}
		table.insert(self.enemies, Enemy:new{x = enemy.x, y = enemy.y, behavior = "patrol", path = path})
	end
	self.noises = {}
	for i, noise in ipairs(prop.noise_list) do
		table.insert(self.noises,SoundMaker:new{x = noise.x, y = noise.y, r = 50})
	end
	self.rubber_ducky = SoundMaker:new{x = prop.ducky_pos.x, y = prop.ducky_pos.y,
					r = 25, color = "yellow"}
	self.door = SoundMaker:new{x = prop.door_pos.x, y = prop.door_pos.y,
					r = 50, color = "green"}

	self.timer = 0
end


--[[
	Performed to destroy the game state (eg. moving from level-to-level)
]]
function Game:stop() end


--[[
	Performed to update the game state (each step it's active)
]]
function Game:update(dt)
	local input = InputManager
	if input:wasPressed("back") then
		StateManager:pop()
	end
	if input:wasPressed("action") then
		if not self.player:gotDucky() then
			self.player:setDucky(self.rubber_ducky:obtain(self.player.x, self.player.y))
		end
		if self.door:obtain(self.player.x, self.player.y) and self.player:gotDucky() then
			if self.level:lastMap() then
				StateManager:push("win")
			else
				StateManager:push("game", self.level.option + 1)
			end
		end
	end
	for i,v in ipairs(self.enemies) do
		v:update(dt)
	end
	self.player:update(dt)
	for i, waves in ipairs(self.player:getWavesCollision()) do
		for key, particles in ipairs(waves) do
			for j, enemy in ipairs(self.enemies) do
				if Hit:check(particles, enemy:getCollisionShape()) then
					local x, y = self.player:getPosition()
					enemy:heardNoise(x, y)
					self.timer = 120
				end
			end
		end
	end
	if self.timer < 0 then
		for i, enemy in ipairs(self.enemies) do
			enemy:noNoise()
		end
	elseif self.timer > 0 then
		self.timer = self.timer - 10*dt
	end
	for i,noise in ipairs(self.noises) do
		noise:update(dt)
		noise:objWithinRadius(self.player:getCollisionShape())
	end
	if not self.player:gotDucky() then
		self.rubber_ducky:update(dt)
		self.rubber_ducky:objWithinRadius(self.player:getCollisionShape())
	end
	self.door:update(dt)
	self.door:objWithinRadius(self.player:getCollisionShape())
end





return Game
