--[[
	ColorTool controller
	Caveware Digital :: 2017
]]


local ColorTool = {
	_list = require(... .. "/color_list")
}


--[[
	Blends two colors
]]
function ColorTool:blend(color_1, color_2, distance)
	-- Load colors from the internal list
	color_1 = self:get(color_1)
	color_2 = self:get(color_2)

	-- Add fourth channel if not supplied, and default distance to 0.5
	color_1[4] = color_1[4] or 255
	color_2[4] = color_2[4] or 255
	distance = distance or 0.5

	-- Blend colors
	local blended_color = {}
	for i = 1, 4 do
		local difference = color_2[i] - color_1[i]
		blended_color[i] = color_1[i] + difference * distance
	end

	return blended_color
end


--[[
	Retrieves a color from the internal list given a key
	INPUT: color - The key of the color from the internal list or color table
	OUTPUT: ColorTool retrieved from the list
]]
function ColorTool:get(color)
	return type(color) == "string" and self._list[color] or color
end


return ColorTool
