--[[
	Hit detector
	Caveware Digital :: 2017
]]


local Hit = {}


--[[
	CIRCLES
	-------
	Require three properties:
	> x, y :: Centre point of circle
	> r :: Radius of circle

	RECTANGLES
	----------
	Require four properties
	> x, y :: Top-left corner of rectangle
	> w, h :: Width and height of rectangle
]]


--[[
	Accepts two circles/rectangles/ and checks if they're colliding
	INPUT: Two objects
	OUTPUT: Whether they're touching or not
]]
function Hit:check(object1, object2)
	-- Account for if both objects are circles
	if (object1.r and object2.r) then
		return self:checkCircles(object1, object2)
	-- Account for mixed objects
	elseif (object1.r or object2.r) then
		if (object2.r) then
			return self:checkMixed(object2, object1)
		end
		return self:checkMixed(object1, object2)
	-- Account for checking two rectangles
	else
		return self:checkRectangles(object1, object2)
	end
end


--[[
	Checks collision between two circles
	INPUT: Two circles
	OUTPUT: Whether they're touching or not
]]
function Hit:checkCircles(circle1, circle2)
	local distance = math.distance(circle1.x, circle1.y, circle2.x, circle2.y)
	return (circle1.r + circle2.r) > distance
end


--[[
	Checks collision between a circle and a rectangle, in that order
	INPUT: Circle and rectangle
	OUTPUT: Whether they're touching or not
]]
function Hit:checkMixed(circle, rect)
	local point = {
		x = math.clamp(rect.x, circle.x, rect.x + rect.w),
		y = math.clamp(rect.y, circle.y, rect.y + rect.h)
	}

	return math.distance(circle.x, circle.y, point.x, point.y) < circle.r
end


--[[
	Check collision between two rectangles
	INPUT: Two rectangles
	OUTPUT: Whether they're touching or not
]]
function Hit:checkRectangles(rect1, rect2)
	return rect1.x + rect1.w > rect2.x and
	       rect1.y + rect1.h > rect2.y and
	       rect1.x < rect2.x + rect2.w and
	       rect1.y < rect2.y + rect2.h
end

--[[
	Check collision between
]]

return Hit
